==> druid-detector-0.6.tar.gz: druid-detector-0.6/app/detector.py <==
# vi: set softtabstop=2 ts=2 sw=2 expandtab:
# pylint:

from inspect import isclass
from . import logger

# ---------------------------------------------------------------------------
#                                                          DetectorRegistry
# ---------------------------------------------------------------------------

class DetectorRegistry():

  _instance = None

  def __new__(cls):
    if cls._instance is None:
      cls._instance = super(DetectorRegistry, cls).__new__(cls)
    return cls._instance

  # TODO: though this is a singleton, will call __init__() on every __new__()
  # even though we don't create a new instance.  So, the instance is re-
  # initialized.  We could get around this by initializing in __new__ but that
  # seems lame.  As well, originally there was a get_registry() class method
  # that avoided all this, but the problem is, Pylint fails to recognize that
  # returns an instance of DetectorRegistry, and then complains when iterating
  # over it that it's not an iterator.
  def __init__(self):
    self._detectors = {}

  def __iter__(self):
    for detector in self._detectors.values():
      yield detector

  def register(self, detector):
    """
    Register a Detector implementation.

    Subclasses of the `Detector` class must register themselves so the
    application knows to query these classes.

    Args:
      name: The common name for the detector type.  This name will be used,
        for example, when reporting via the API.  It is recommended it be
        plural, for example, "bursts" or "oldjobs".
      detector: The subclass.

    Raises:
      `manager.exceptions.AppException` if a detector with that name has
        already been registered.
    """
    name = detector.name

    if name in self._detectors:
      raise Exception(f"A detector has already been registered with that name: {name}")
    if not isclass(detector) or not issubclass(detector, Detector):
      raise Exception("The detector is not a subclass of Detector")

    # register
    self._detectors[name] = detector
    logger.info("Registered detector for %s", name)

# create single instance of registry
registry = DetectorRegistry()

# ---------------------------------------------------------------------------
#                                                                  Detector
# ---------------------------------------------------------------------------

class Detector():

  _name = None
  _config = None

  @classmethod
  @property
  def name(cls):
    return cls._name

  @classmethod
  def define_configuration(cls, config):
    """
    Given a MergeConfSection object, define configuration items for the
    application configuration which will be later merged and made available.

    Arguments:
      config: A MergeConfSection object defined for this Detector class.
    """
    # keep config for later
    cls._config = config

    # add `enable` item for all Detectors
    config.add('enable', type=bool, value=False, mandatory=True)

  @classmethod
  def detect(cls):
    """
    Detect the things and report the things.

    This method handles detection of reportable cases and compiles a list to
    contribute to the overall report.
    ```
      [
        {                     # this is a list of dicts describing each case.
          'account': ...,     # here is a datum reported for all case types
          'resource': ...,    # here is a datum specific to this type of case
          'summary': ...,     # all cases support a summary, though optional
          ...                 # there are definitely others
        },
        ...                   # this list can be one, many, or zero length
      ],
    ```
    """
    raise NotImplementedError()

==> druid-detector-0.6.1.tar.gz: druid-detector-0.6.1/app/detector.py <==
# vi: set softtabstop=2 ts=2 sw=2 expandtab:
# pylint:

from inspect import isclass
from . import logger

# ---------------------------------------------------------------------------
#                                                          DetectorRegistry
# ---------------------------------------------------------------------------

class DetectorRegistry():

  _instance = None

  def __new__(cls):
    if cls._instance is None:
      cls._instance = super(DetectorRegistry, cls).__new__(cls)
    return cls._instance

  # TODO: though this is a singleton, will call __init__() on every __new__()
  # even though we don't create a new instance.  So, the instance is re-
  # initialized.  We could get around this by initializing in __new__ but that
  # seems lame.  As well, originally there was a get_registry() class method
  # that avoided all this, but the problem is, Pylint fails to recognize that
  # returns an instance of DetectorRegistry, and then complains when iterating
  # over it that it's not an iterator.
  def __init__(self):
    self._detectors = {}

  def __iter__(self):
    for detector in self._detectors.values():
      yield detector

  def register(self, detector):
    """
    Register a Detector implementation.

    Subclasses of the `Detector` class must register themselves so the
    application knows to query these classes.

    Args:
      name: The common name for the detector type.  This name will be used,
        for example, when reporting via the API.  It is recommended it be
        plural, for example, "bursts" or "oldjobs".
      detector: The subclass.

    Raises:
      `manager.exceptions.AppException` if a detector with that name has
        already been registered.
    """
    name = detector.name

    if name in self._detectors:
      raise Exception(f"A detector has already been registered with that name: {name}")
    if not isclass(detector) or not issubclass(detector, Detector):
      raise Exception("The detector is not a subclass of Detector")

    # register
    self._detectors[name] = detector
    logger.info("Registered detector for %s", name)

# create single instance of registry
registry = DetectorRegistry()

# ---------------------------------------------------------------------------
#                                                                  Detector
# ---------------------------------------------------------------------------

class Detector():

  _name = None
  _config = None

  @classmethod
  @property
  def name(cls):
    return cls._name

  @classmethod
  def define_configuration(cls, config):
    """
    Given a MergeConfSection object, define configuration items for the
    application configuration which will be later merged and made available.

    Arguments:
      config: A MergeConfSection object defined for this Detector class.
    """
    # keep config for later
    cls._config = config

    # add `enable` item for all Detectors
    config.add('enable', type=bool, value=False, mandatory=True,
      description=f"Enable the {cls.__name__} detector")

  @classmethod
  def detect(cls):
    """
    Detect the things and report the things.

    This method handles detection of reportable cases and compiles a list to
    contribute to the overall report.
    ```
      [
        {                     # this is a list of dicts describing each case.
          'account': ...,     # here is a datum reported for all case types
          'resource': ...,    # here is a datum specific to this type of case
          'summary': ...,     # all cases support a summary, though optional
          ...                 # there are definitely others
        },
        ...                   # this list can be one, many, or zero length
      ],
    ```
    """
    raise NotImplementedError()
